"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.S3FileSaver = void 0;
const common_1 = require("@nestjs/common");
const client_s3_1 = require("@aws-sdk/client-s3");
class S3FileSaver {
    /**
     * Initializes a new instance of the S3FileSaver class.
     * Sets up the S3 client using the provided configuration options.
     * @param fileUploadOptions - configuration options for S3 file uploads.
     */
    constructor(fileUploadOptions) {
        this.fileUploadOptions = fileUploadOptions;
        this.s3Client = new client_s3_1.S3Client(fileUploadOptions);
    }
    /**
     * Saves the provided file data to an S3 bucket.
     * Constructs the S3 object parameters and sends a PutObjectCommand to S3.
     * @param fileData - The file data to be saved, including the buffer and metadata.
     * @param context - The execution context, typically provided by NestJS.
     * @param options - Optional S3-specific file data options, such as bucket and additional parameters.
     * @returns A promise that resolves to the URL of the saved file in S3.
     * @throws InternalServerErrorException if the S3 bucket is not specified.
     */
    async save(fileData, context, options) {
        const params = {
            Bucket: options?.Bucket ?? this.fileUploadOptions?.bucket,
            Key: fileData.fileNameFull,
            Body: fileData.buffer,
            ContentType: fileData.mimetype,
            ...options,
        };
        if (!params?.Bucket) {
            throw new common_1.InternalServerErrorException(`Bucket is required`);
        }
        const command = new client_s3_1.PutObjectCommand(params);
        await this.s3Client.send(command);
        const s3Endpoint = await this.s3Client.config.endpoint();
        const fileUrl = this.fileUploadOptions.endpoint
            ? `${this.fileUploadOptions.endpoint}${s3Endpoint.path}${params.Bucket}/${params.Key}`
            : `https://${params.Bucket}.s3${this.fileUploadOptions.region}.amazonaws.com/${params.Key}`;
        return fileUrl;
    }
}
exports.S3FileSaver = S3FileSaver;
//# sourceMappingURL=s3.file-saver.js.map