"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FormdataInterceptor = void 0;
const common_1 = require("@nestjs/common");
const rxjs_1 = require("rxjs");
const busboy_1 = __importDefault(require("busboy"));
const FileData_1 = require("../classes/FileData");
const defaultInterceptor_config_1 = require("../config/defaultInterceptor.config");
const crypto_1 = __importDefault(require("crypto"));
/**
 * Interceptor to handle file uploads using Busboy.
 */
let FormdataInterceptor = class FormdataInterceptor {
    /**
     * Constructs a new instance of the FormdataInterceptor class.
     * @param fileOptions - Optional configuration options for file handling.
     */
    constructor(fileOptions) {
        this.fileOptions = fileOptions;
        this.arrayRegexPattern = /\[\]$/;
        this.nestedRegexPattern = /[[\]]/;
        this.fileOptions = { ...defaultInterceptor_config_1.DEFAULT_INTERCEPTOR_CONFIG, ...fileOptions };
    }
    /**
     * Intercepts the request to handle file uploads if the content type is multipart/form-data.
     * @param context - The execution context.
     * @param next - The next call handler.
     * @returns An Observable that processes the file upload.
     */
    async intercept(context, next) {
        const { customFileName, fileSaver } = this.fileOptions;
        const ctx = context.switchToHttp();
        this.httpRequest = ctx.getRequest();
        const request = this.httpRequest.raw ?? this.httpRequest;
        const contentType = request.headers["content-type"];
        if (contentType?.includes("multipart/form-data")) {
            return this.handleMultipartFormData(context, next, customFileName, fileSaver);
        }
        return next.handle();
    }
    /**
     * Handles multipart/form-data file uploads.
     * @param context - The execution context.
     * @param next - The next call handler.
     * @param request - The HTTP request.
     * @param customFileName - Optional function to customize file names.
     * @param fileSaver - Optional file saver implementation.
     * @returns An Observable that processes the file upload.
     */
    async handleMultipartFormData(context, next, customFileName, fileSaver) {
        return new rxjs_1.Observable((observer) => {
            this.busboy = (0, busboy_1.default)({ headers: this.httpRequest.headers });
            const files = {};
            const fields = {};
            this.busboy.on("file", async (fieldName, fileStream, fileInfo) => {
                const bufferChunks = [];
                let fileSize = 0;
                fileStream.on("data", (data) => {
                    bufferChunks.push(data);
                    fileSize += data.length;
                });
                fileStream.on("end", async () => {
                    if (fileInfo.filename === undefined && fileInfo.mimeType === "application/octet-stream") {
                        this.handleField(files, fieldName, undefined);
                        return;
                    }
                    const hash = crypto_1.default.createHash("md5");
                    const fileBuffer = Buffer.concat(bufferChunks);
                    const fileExtension = fileInfo.filename.split(".").pop();
                    const fileNameOnly = fileInfo.filename
                        .split(".")
                        .slice(0, -1)
                        .join(".");
                    const finalFileName = customFileName
                        ? await customFileName(context, fileNameOnly)
                        : fileNameOnly;
                    const fullFileName = `${finalFileName}.${fileExtension}`;
                    this.assignFile(fileSaver, context);
                    const fileData = new FileData_1.FileData(fileInfo.filename, finalFileName, fullFileName, fileInfo.encoding, fileInfo.mimeType, fileExtension, fileSize, hash.update(fileBuffer).digest("hex"), fileBuffer);
                    this.handleField(files, fieldName, fileData);
                });
                fileStream.on("error", (error) => {
                    observer.error(error);
                    this.handleDone();
                });
            });
            this.busboy.on("field", (fieldName, val) => {
                this.handleField(fields, fieldName, val);
            });
            this.busboy.on("finish", () => {
                this.httpRequest["body"] = { ...fields };
                this.httpRequest[this.fileOptions.requestFileLocation] = {
                    ...this.httpRequest[this.fileOptions.requestFileLocation],
                    ...files,
                };
                next.handle().subscribe({
                    next: (val) => observer.next(val),
                    error: (error) => observer.error(error),
                    complete: () => {
                        observer.complete();
                        this.handleDone();
                    },
                });
            });
            this.httpRequest?.raw
                ? this.httpRequest.raw.pipe(this.busboy)
                : this.httpRequest.pipe(this.busboy);
        });
    }
    /**
     * Cleans up the resources used by the `busboy` instance and removes all event listeners.
     */
    handleDone() {
        this.busboy.removeAllListeners();
        this.httpRequest.raw
            ? this.httpRequest.raw.unpipe(this.busboy)
            : this.httpRequest.unpipe(this.busboy);
    }
    /**
     * assign save file strategy
     *
     * @param fileSaver  - file saver  strategy
     * @param context - The execution context.
     */
    assignFile(fileSaver, context) {
        FileData_1.FileData.prototype.save = function (args) {
            return fileSaver.save(this, context, args);
        };
    }
    /**
     * Handles a field by setting its value in the target object, supporting nested fields and arrays.
     * @param target - The target object to set the field value.
     * @param fieldName - The name of the field.
     * @param value - The value to set.
     */
    handleField(target, fieldName, value) {
        const keys = fieldName.split(this.nestedRegexPattern).filter(Boolean);
        const isArrayField = this.arrayRegexPattern.test(fieldName);
        this.setNestedValue(target, keys, value, isArrayField);
    }
    /**
     * Sets a nested value in an object, supporting nested fields and arrays.
     * @param obj - The object to set the nested value in.
     * @param keys - The keys representing the nested path.
     * @param value - The value to set.
     * @param isArray - Whether the field is an array.
     */
    setNestedValue(obj, keys, value, isArray = false) {
        let current = obj;
        keys.forEach((key, index) => {
            const isLastKey = index === keys.length - 1;
            if (isLastKey) {
                this.assignValue(current, key, value, isArray);
            }
            else {
                if (!current[key]) {
                    current[key] = {};
                }
                current = current[key];
            }
        });
    }
    /**
     * Assigns a value to the specified key in an object, supporting arrays.
     * @param obj - The object to set the value in.
     * @param key - The key to set the value for.
     * @param value - The value to set.
     * @param isArray - Whether the field is an array.
     */
    assignValue(obj, key, value, isArray) {
        isArray
            ? this.assignArrayValue(obj, key, value)
            : this.assignSingleValue(obj, key, value);
    }
    /**
     * Assigns a single value or array value to the specified key in an object.
     * @param obj - The object to set the value in.
     * @param key - The key to set the value for.
     * @param value - The value to set.
     */
    assignSingleValue(obj, key, value) {
        if (obj[key]) {
            if (Array.isArray(obj[key])) {
                obj[key].push(value);
            }
            else {
                obj[key] = [obj[key], value];
            }
        }
        else {
            obj[key] = value;
        }
    }
    /**
     * Assigns a value to an array at the specified key in an object.
     * @param obj - The object to set the value in.
     * @param key - The key to set the value for.
     * @param value - The value to set.
     */
    assignArrayValue(obj, key, value) {
        if (!Array.isArray(obj[key])) {
            obj[key] = [];
        }
        obj[key].push(value);
    }
};
exports.FormdataInterceptor = FormdataInterceptor;
exports.FormdataInterceptor = FormdataInterceptor = __decorate([
    (0, common_1.Injectable)(),
    __metadata("design:paramtypes", [Object])
], FormdataInterceptor);
//# sourceMappingURL=formdata.interceptor.js.map