"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const local_file_saver_1 = require("../../fileSaver/local.file-saver");
const file_interface_1 = require("../../interfaces/file.interface");
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const FileData_1 = require("../../classes/FileData");
// Mock the fs module
jest.mock("fs");
describe("LocalFileSaver", () => {
    let fileSaver;
    const mockPrefixDirectory = "./public";
    const mockExecutionContext = {};
    const mockFileData = new FileData_1.FileData("file.txt", "file", "file.txt", "7bit", file_interface_1.MimeType["image/png"], "txt", 100, "hash", Buffer.from("Hello, world!"));
    beforeEach(() => {
        const options = {
            prefixDirectory: mockPrefixDirectory,
        };
        fileSaver = new local_file_saver_1.LocalFileSaver(options);
    });
    afterEach(() => {
        jest.clearAllMocks();
    });
    it("should create directory if it does not exist and save file", () => {
        const mockFilePath = path_1.default
            .join(mockPrefixDirectory, mockFileData.fileNameFull)
            .replace(/\\/g, "/");
        // Mock implementations for fs functions
        fs_1.default.existsSync.mockReturnValue(false);
        fs_1.default.mkdirSync.mockReturnValue(undefined);
        fs_1.default.writeFileSync.mockReturnValue(undefined);
        const result = fileSaver.save(mockFileData, mockExecutionContext);
        expect(fs_1.default.existsSync).toHaveBeenCalledWith(mockPrefixDirectory.replace(/^\.\//, ""));
        expect(fs_1.default.mkdirSync).toHaveBeenCalledWith(mockPrefixDirectory.replace(/^\.\//, ""), {
            recursive: true,
        });
        expect(fs_1.default.writeFileSync).toHaveBeenCalledWith(mockFilePath.replace(/^\.\//, ""), mockFileData.buffer);
        expect(result).toBe(mockFilePath);
    });
    it("should save file in subdirectory when options.path is provided", () => {
        const subPath = "sub/folder";
        const fullDirectory = path_1.default.join(mockPrefixDirectory, subPath);
        const mockFilePath = path_1.default
            .join(fullDirectory, mockFileData.fileNameFull)
            .replace(/\\/g, "/");
        fs_1.default.existsSync.mockReturnValue(false);
        fs_1.default.mkdirSync.mockReturnValue(undefined);
        fs_1.default.writeFileSync.mockReturnValue(undefined);
        const result = fileSaver.save(mockFileData, mockExecutionContext, {
            path: subPath,
        });
        expect(fs_1.default.existsSync).toHaveBeenCalledWith(fullDirectory.replace(/\\/g, "/"));
        expect(fs_1.default.mkdirSync).toHaveBeenCalledWith(fullDirectory.replace(/\\/g, "/"), {
            recursive: true,
        });
        expect(fs_1.default.writeFileSync).toHaveBeenCalledWith(mockFilePath, mockFileData.buffer);
        expect(result).toBe(mockFilePath);
    });
    it("should save file when directory already exists", () => {
        const mockFilePath = path_1.default
            .join(mockPrefixDirectory, mockFileData.fileNameFull)
            .replace(/\\/g, "/");
        // Mock implementations for fs functions
        fs_1.default.existsSync.mockReturnValue(true);
        fs_1.default.writeFileSync.mockReturnValue(undefined);
        const result = fileSaver.save(mockFileData, mockExecutionContext);
        expect(fs_1.default.existsSync).toHaveBeenCalledWith(mockPrefixDirectory.replace(/^\.\//, ""));
        expect(fs_1.default.mkdirSync).not.toHaveBeenCalled();
        expect(fs_1.default.writeFileSync).toHaveBeenCalledWith(mockFilePath, mockFileData.buffer);
        expect(result).toBe(mockFilePath);
    });
    it("should use custom directory function if provided", () => {
        const customDirectory = jest.fn().mockReturnValue("custom/dir");
        const options = {
            prefixDirectory: mockPrefixDirectory,
            customDirectory,
        };
        fileSaver = new local_file_saver_1.LocalFileSaver(options);
        const mockFilePath = path_1.default
            .join("custom/dir", mockFileData.fileNameFull)
            .replace(/\\/g, "/");
        // Mock implementations for fs functions
        fs_1.default.existsSync.mockReturnValue(false);
        fs_1.default.mkdirSync.mockReturnValue(undefined);
        fs_1.default.writeFileSync.mockReturnValue(undefined);
        const result = fileSaver.save(mockFileData, mockExecutionContext);
        expect(customDirectory).toHaveBeenCalledWith(mockExecutionContext, mockPrefixDirectory);
        expect(fs_1.default.existsSync).toHaveBeenCalledWith("custom/dir");
        expect(fs_1.default.mkdirSync).toHaveBeenCalledWith("custom/dir", {
            recursive: true,
        });
        expect(fs_1.default.writeFileSync).toHaveBeenCalledWith(mockFilePath, mockFileData.buffer);
        expect(result).toBe(mockFilePath);
    });
});
//# sourceMappingURL=localFileSaver.spec.js.map