"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const s3_file_saver_1 = require("../../fileSaver/s3.file-saver");
const file_interface_1 = require("../../interfaces/file.interface");
const common_1 = require("@nestjs/common");
const FileData_1 = require("../../classes/FileData");
const client_s3_1 = require("@aws-sdk/client-s3");
// Mock AWS SDK
jest.mock("@aws-sdk/client-s3");
describe("S3FileSaver", () => {
    let fileSaver;
    let mockS3ClientSend;
    const mockExecutionContext = {};
    const mockFileData = new FileData_1.FileData("test.jpg", "field", "test.jpg", "7bit", file_interface_1.MimeType["image/jpeg"], "jpg", 123, "hash", Buffer.from("mock content"));
    const defaultOptions = {
        region: "us-east-1",
        bucket: "mock-bucket",
        endpoint: "https://custom-endpoint.com/",
    };
    beforeEach(() => {
        mockS3ClientSend = jest.fn().mockResolvedValue({});
        client_s3_1.S3Client.mockImplementation(() => ({
            send: mockS3ClientSend,
            config: {
                endpoint: jest.fn().mockResolvedValue({ path: "/" }),
            },
        }));
        client_s3_1.PutObjectCommand.mockImplementation((params) => ({
            mockCommand: true,
            input: params,
        }));
        fileSaver = new s3_file_saver_1.S3FileSaver(defaultOptions);
    });
    afterEach(() => {
        jest.clearAllMocks();
    });
    it("should upload file to S3 and return custom URL", async () => {
        const result = await fileSaver.save(mockFileData, mockExecutionContext);
        expect(client_s3_1.PutObjectCommand).toHaveBeenCalledWith(expect.objectContaining({
            Bucket: "mock-bucket",
            Key: "test.jpg",
            Body: mockFileData.buffer,
            ContentType: "image/jpeg",
        }));
        expect(mockS3ClientSend).toHaveBeenCalledWith(expect.objectContaining({ mockCommand: true }));
        expect(result).toBe("https://custom-endpoint.com//mock-bucket/test.jpg");
    });
    it("should throw an exception if bucket is missing", async () => {
        const noBucketOptions = {
            region: "us-east-1",
        };
        fileSaver = new s3_file_saver_1.S3FileSaver(noBucketOptions);
        await expect(fileSaver.save(mockFileData, mockExecutionContext)).rejects.toThrow(common_1.InternalServerErrorException);
    });
    it("should use Bucket from save() options if provided", async () => {
        const saveOptions = {
            Bucket: "alternate-bucket",
        };
        await fileSaver.save(mockFileData, mockExecutionContext, saveOptions);
        expect(client_s3_1.PutObjectCommand).toHaveBeenCalledWith(expect.objectContaining({
            Bucket: "alternate-bucket",
        }));
    });
    it("should return default AWS S3 URL if no endpoint is provided", async () => {
        const fallbackOptions = {
            region: "us-west-2",
            bucket: "backup-bucket",
        };
        fileSaver = new s3_file_saver_1.S3FileSaver(fallbackOptions);
        const result = await fileSaver.save(mockFileData, mockExecutionContext);
        expect(result).toBe("https://backup-bucket.s3us-west-2.amazonaws.com/test.jpg");
    });
});
//# sourceMappingURL=s3FileSaver.spec.js.map